/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 12 : Multiple Instances witout Synchronization
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * In this example we have an activity called 'order_books' that simulates
 * a book order. It instantiates three sub processes to manage each book.
 * These sub process are independent of next activity 'send_bill' and 
 * between them.
--------------------------------------------------------------------------
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern12MultipleInstancesWithoutSynchronization {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 12 - Multiple Instances without Synchronization");
			prjSession.initModel("Pattern 12 - Multiple Instances without Synchronization");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("order_books", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("send_bill", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.setNodeTraditional("order_books");
			prjSession.setNodeTraditional("send_bill");
			prjSession.addEdge("order_books", "send_bill");
			
			// Add Project Properties and set transition conditions
			prjSession.setProperty("books", "3");
			
			// SubProcess definition
			ProjectSession prjSession2 = prjHome.create();
			prjSession2.initModel("Process book order (SubProcess)");
			prjSession2.addNode("update_stock",Constants.Nd.AND_JOIN_NODE);
			prjSession2.addNode("send_book",Constants.Nd.AND_JOIN_NODE);
			prjSession2.setNodeTraditional("update_stock");
			prjSession2.setNodeTraditional("send_book");
			prjSession2.addEdge("update_stock", "send_book");
			String update_stockHookScript =
				  "import hero.interfaces.*;\n"
				+ "beforeStart(Object engine, Object currentNode) {\n\n\n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(currentNode.getBnProject().getName()); \n"
				+ "  BnProjectPropertyValue bookNameProp = prjSession.getProperty(\"book_name\"); \n"
				+ "  System.out.println(\"   ---> update_stock : \" + bookNameProp.getTheValue()); \n"
				+ "}";			
			prjSession2.addNodeInterHook("update_stock", "update_stock Hook", hero.interfaces.Constants.Nd.BEFORESTART, hero.interfaces.Constants.Hook.BSINTERACTIVE, update_stockHookScript);			
			prjSession2.addRole("Executor", "Rol that enables to execute the activities inside the SubProcess");
			prjSession2.setUserRole("admin", "Executor");
			prjSession2.setNodeRole("update_stock", "Executor");
			prjSession2.setNodeRole("send_book", "Executor");
			prjSession2.remove();
			
			// Add Hook to 'Order books' activity to create a SubProces for each book 
			String approveHookScript =
				  "import hero.interfaces.*;\n"
				+ "beforeTerminate(Object engine, Object currentNode) {\n\n\n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(currentNode.getBnProject().getName()); \n"
				+ "  BnProjectPropertyValue booksProp = prjSession.getProperty(\"books\"); \n"
				+ "  int books = Integer.parseInt(booksProp.getTheValue()); \n"
				+ "  for (int i = 0; i < books; i++) { \n"
				+ "    String nodeName = \"book\" + Integer.toString(i); \n"
				+ "    prjSession.addNodeSubProcess(nodeName, \"Process book order (SubProcess)\"); \n"
				+ "    prjSession.setNodeProperty(nodeName, \"book_name\", nodeName, true); \n"
				+ "    prjSession.addEdge(\"order_books\", nodeName); \n"
				+ "    prjSession.setNodeRole(nodeName, \"Executor\"); \n"
				+ "  } \n"
				+ "}";
			prjSession.addNodeInterHook("order_books", "Order books Hook", hero.interfaces.Constants.Nd.BEFORETERMINATE, hero.interfaces.Constants.Hook.BSINTERACTIVE, approveHookScript);
			
			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("order_books", "Executor");
			prjSession.setNodeRole("send_bill", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
