/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 16 : Deferred Choice
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * In this example we have two activities avaliable: load_truck1 and 
 * load_truck2. select_avaliable_resource activity has hook that decides 
 * which truck is avaliable (the decision is taken at random using a hook) 
--------------------------------------------------------------------------
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern16DeferredChoice {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 16 - Deferred Choice");
			prjSession.initModel("Pattern 16 - Deferred Choice");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("select_avaliable_resource", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("load_truck1", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("load_truck2", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.setNodeTraditional("select_avaliable_resource");
			prjSession.setNodeTraditional("load_truck1");
			prjSession.setNodeTraditional("load_truck2");
			
			// Add transitions between activities
			String toTruck1 = prjSession.addEdge("select_avaliable_resource", "load_truck1");
			String toTruck2 = prjSession.addEdge("select_avaliable_resource", "load_truck2");

			// Add a Project Property and set transition conditions
			prjSession.setProperty("truck_number", "0");
			prjSession.setEdgeCondition(toTruck1, "truck_number.equals(\"1\")");
			prjSession.setEdgeCondition(toTruck2, "truck_number.equals(\"2\")");
			
			// Set a hook that simulates trucks avaliability 
			String simulateAvalibilityHookScript =
				  "import hero.interfaces.*;\n"
				+ "import java.util.Random; \n"
				+ "beforeTerminate(Object engine, Object currentNode) {\n\n\n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(currentNode.getBnProject().getName()); \n"				
			    + "  Random generator = new Random(System.currentTimeMillis()); \n"
			    + "  int truck = generator.nextInt(2) + 1; \n"
			    + "  prjSession.setProperty(\"truck_number\", Integer.toString(truck)); \n"
			    + "  System.out.println(\"    --> Truck avaliable : \" + truck); \n"
				+ "} \n";
			prjSession.addNodeInterHook("select_avaliable_resource", "select_avaliable_resource hook", hero.interfaces.Constants.Nd.BEFORETERMINATE, hero.interfaces.Constants.Hook.BSINTERACTIVE, simulateAvalibilityHookScript);			

			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("select_avaliable_resource", "Executor");
			prjSession.setNodeRole("load_truck1", "Executor");
			prjSession.setNodeRole("load_truck2", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
