/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 17 : Interleaved Parallel Routing
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * In this example we have 3 activies: physical_test, mental_test and
 * skill_test that we want to execute but not at the same time.
 * We will use an iteration to execute the three activities and after
 * that continues the process execution.
--------------------------------------------------------------------------
*/
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern17InterleavedParallelRouting {

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 17 - Interleaved Parallel Routing");
			prjSession.initModel("Pattern 17 - Interleaved Parallel Routing");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("select_avaliable_test", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("physical_test", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("mental_test", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("skill_test", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("decide", hero.interfaces.Constants.Nd.OR_JOIN_NODE);
			prjSession.addNode("tests_report", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.setNodeTraditional("select_avaliable_test");
			prjSession.setNodeTraditional("physical_test");
			prjSession.setNodeTraditional("mental_test");
			prjSession.setNodeTraditional("skill_test");
			prjSession.setNodeTraditional("decide");
			prjSession.setNodeTraditional("tests_report");
			
			// Add transitions between activities
			String toPhysicalTest = prjSession.addEdge("select_avaliable_test", "physical_test");
			String toMentalTest = prjSession.addEdge("select_avaliable_test", "mental_test");
			String toSkillTest = prjSession.addEdge("select_avaliable_test", "skill_test");
			prjSession.addEdge("physical_test", "decide");
			prjSession.addEdge("mental_test", "decide");
			prjSession.addEdge("skill_test", "decide");
			String toTestReport = prjSession.addEdge("decide", "tests_report");
			
			// Add a Project Property and set transition conditions
			prjSession.setProperty("test", "");
			prjSession.setProperty("physical_test", "not_passed");
			prjSession.setProperty("mental_test", "not_passed");
			prjSession.setProperty("skill_test", "not_passed");
			prjSession.setEdgeCondition(toPhysicalTest, "test.equals(\"1\")");
			prjSession.setEdgeCondition(toMentalTest, "test.equals(\"2\")");
			prjSession.setEdgeCondition(toSkillTest, "test.equals(\"3\")");
			prjSession.setEdgeCondition(toTestReport, "physical_test.equals(\"passed\") && mental_test.equals(\"passed\") && skill_test.equals(\"passed\")");
			
			// Add hook that simulates the avaliability of the tests
			String simulateAvalibilityHookScript =
				  "import hero.interfaces.*;\n"
				+ "import java.util.Random; \n"
				+ "beforeTerminate(Object engine, Object currentNode) { \n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(currentNode.getBnProject().getName()); \n"
				+ "  String physicalPassed = prjSession.getProperty(\"physical_test\").getTheValue(); \n"				
				+ "  String mentalPassed = prjSession.getProperty(\"mental_test\").getTheValue(); \n"				
				+ "  String skillPassed = prjSession.getProperty(\"skill_test\").getTheValue(); \n"				
			    + "  Random generator = new Random(System.currentTimeMillis()); \n"
			    + "  boolean avaliableTest = false; \n"
			    + "  int testNum; \n"
			    + "  System.out.println(\" Physical : \" + physicalPassed); \n"
			    + "  System.out.println(\"   Mental : \" + mentalPassed); \n"
			    + "  System.out.println(\"    Skill : \" + skillPassed); \n"
			    + "  while (avaliableTest == false) { \n"
			    + "    testNum = generator.nextInt(3) + 1; \n"
			    + "    System.out.println(\"    --> Test : \" + testNum); \n"
			    + "    if (((testNum == 1) && (physicalPassed.equals(\"not_passed\")))" 
			    + "        || ((testNum == 2) && (mentalPassed.equals(\"not_passed\")))"
			    + "        || ((testNum == 3) && (skillPassed.equals(\"not_passed\")))) { \n"
			    + "      avaliableTest = true; \n"
			    + "    } \n"
			    + "  } \n"
			    + "  prjSession.setProperty(\"test\", Integer.toString(testNum)); \n"
			    + "  System.out.println(\"    --> Tests remaining : \" + testNum); \n"
				+ "} \n";
			prjSession.addNodeInterHook("select_avaliable_test", "Select test Hook", hero.interfaces.Constants.Nd.BEFORETERMINATE, hero.interfaces.Constants.Hook.BSINTERACTIVE, simulateAvalibilityHookScript);

			// Add hook that set each test as passed
			String testPassedHookScript =
				  "import hero.interfaces.*;\n"
				+ "import java.util.Random; \n"
				+ "beforeTerminate(Object engine, Object currentNode) { \n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(currentNode.getBnProject().getName()); \n"
				+ "  String nodeName = currentNode.getName(); \n"
			    + "  prjSession.setProperty(nodeName, \"passed\"); \n"
			    + "  System.out.println(\"    --> Test \" + nodeName + \" : passed\"); \n"
				+ "} \n";
			prjSession.addNodeInterHook("physical_test", "physical_test Hook", hero.interfaces.Constants.Nd.BEFORETERMINATE, hero.interfaces.Constants.Hook.BSINTERACTIVE, testPassedHookScript);
			prjSession.addNodeInterHook("mental_test", "mental_test Hook", hero.interfaces.Constants.Nd.BEFORETERMINATE, hero.interfaces.Constants.Hook.BSINTERACTIVE, testPassedHookScript);
			prjSession.addNodeInterHook("skill_test", "skill_test Hook", hero.interfaces.Constants.Nd.BEFORETERMINATE, hero.interfaces.Constants.Hook.BSINTERACTIVE, testPassedHookScript);

			// Add iteration until all tests are passed
			prjSession.addIteration("decide", "select_avaliable_test", "(physical_test.equals(\"not_passed\") || mental_test.equals(\"not_passed\") || skill_test.equals(\"not_passed\"))");

			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("select_avaliable_test", "Executor");
			prjSession.setNodeRole("physical_test", "Executor");
			prjSession.setNodeRole("mental_test", "Executor");
			prjSession.setNodeRole("skill_test", "Executor");
			prjSession.setNodeRole("decide", "Executor");
			prjSession.setNodeRole("tests_report", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
