/**
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
--------------------------------------------------------------------------
 * BONITA Workflow Patterns sample
 *   Pattern 18 : Milestone
 *       Author : Jordi Anguela
 *         Date : 2006/03/01 
--------------------------------------------------------------------------
 * Example
 * This exemple shows how you can use Bonita deadlines to control the period
 * of execution of an activity.
 * 'withdraw_order' is only enabled :
 *    - only after the execution of 'order_book'
 *    - only during 20 seconds
 *    - only if 'shipping_process' activity is not started 
--------------------------------------------------------------------------
 */
package hero.client.samples.patterns.src;

import hero.client.test.SimpleCallbackHandler;
import hero.interfaces.Constants;
import hero.interfaces.ProjectSession;
import hero.interfaces.ProjectSessionHome;
import hero.interfaces.ProjectSessionUtil;

import javax.security.auth.login.LoginContext;

public class Pattern18MilestoneAsDeadline {
	final static long DEADLINE = 1000; // In miliseconds

	public static void main(String[] args) {
		try {
			// User Admin login
			char[] password = { 't', 'o', 't', 'o' };
			SimpleCallbackHandler handler = new SimpleCallbackHandler("admin", password);
			LoginContext lc = new LoginContext("TestClient", handler);
			lc.login();
	
			// Project Session Bean creation
			ProjectSessionHome prjHome = (ProjectSessionHome) ProjectSessionUtil.getHome();
			ProjectSession prjSession = prjHome.create();
	
			// Model creation 
			System.out.print("\n   Pattern 18 - Milestone as Deadline");
			prjSession.initModel("Pattern 18 - Milestone as Deadline");
			
			// Add activities and set them to Non anticipable
			prjSession.addNode("order_book", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("withdraw_order", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.addNode("ship_order", hero.interfaces.Constants.Nd.AND_JOIN_NODE);
			prjSession.setNodeTraditional("order_book");
			prjSession.setNodeTraditional("withdraw_order");
			prjSession.setNodeTraditional("ship_order");
	
			// Add transitions between activities
			prjSession.addEdge("order_book", "ship_order");
			prjSession.addEdge("order_book", "withdraw_order");
			
			// Add Hook that set the Deadline for 'withdraw_order'
			String setDeadlineScript =
				  "import hero.interfaces.*; \n"
				+ "beforeTerminate(Object engine, Object currentNode) { \n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(currentNode.getBnProject().getName()); \n"
				+ "  ArrayList deadlines = new ArrayList(); \n"
				+ "  long date1 = System.currentTimeMillis() + (long)10000; \n"
				+ "  deadlines.add(new Long(date1)); \n"
				+ "  prjSession.setNodeDeadlines(\"withdraw_order\", deadlines); \n"
				+ "}";
			prjSession.addNodeInterHook("order_book", "order_book Hook", hero.interfaces.Constants.Nd.BEFORETERMINATE, hero.interfaces.Constants.Hook.BSINTERACTIVE, setDeadlineScript);
			
			// Add Deadline Hook to 'withdraw_order' to cancel itself when the deadline expires 
			String onDeadlineScript =
				  "import hero.interfaces.*; \n"
				+ "onDeadline(Object engine, Object currentNode) { \n"
				+ "  System.out.println(\"\n\n   --> Deadline hook <--\"); \n"
				+ "  UserSessionHome userHome = UserSessionUtil.getHome(); \n"
				+ "  UserSession userSession = userHome.create(); \n"
				+ "  userSession.cancelActivity(currentNode.getBnProject().getName(), \"withdraw_order\"); \n "
				+ "}";
			//prjSession.addNodeInterHook("withdraw_order", "withdraw_order Hook", hero.interfaces.Constants.Nd.ONDEADLINE, hero.interfaces.Constants.Hook.BSINTERACTIVE, onDeadlineScript);
			prjSession.addNodeHook("withdraw_order", "hero.hook.CancelActivity", hero.interfaces.Constants.Nd.ONDEADLINE, Constants.Hook.JAVA);
			
			// Add Hook to 'ship_order' to cancel the 'order_book if it has not executed yet 
			String cancelScript =
				  "import hero.interfaces.*; \n"
				+ "beforeStart(Object engine, Object currentNode) { \n"
				+ "  hero.interfaces.ProjectSessionHome pHome = (hero.interfaces.ProjectSessionHome) hero.interfaces.ProjectSessionUtil.getHome(); \n"
				+ "  hero.interfaces.ProjectSession prjSession = pHome.create(); \n"
				+ "  prjSession.initModel(currentNode.getBnProject().getName()); \n"
				+ "  if (prjSession.getNodeState(\"withdraw_order\") == hero.interfaces.Constants.Nd.READY) { \n"
				+ "    UserSessionHome userHome = UserSessionUtil.getHome(); \n"
				+ "    UserSession userSession = userHome.create(); \n"
				+ "    userSession.cancelActivity(currentNode.getBnProject().getName(), \"withdraw_order\"); \n "
				+ "  } \n"
				+ "}";
			prjSession.addNodeInterHook("ship_order", "ship_order Hook", hero.interfaces.Constants.Nd.BEFORESTART, hero.interfaces.Constants.Hook.BSINTERACTIVE, cancelScript);
			
			// Add and set a role for the user admin to execute the activities
			prjSession.addRole("Executor", "Rol that enables to execute the activities");
			prjSession.setUserRole("admin", "Executor");
			prjSession.setNodeRole("order_book", "Executor");
			prjSession.setNodeRole("withdraw_order", "Executor");
			prjSession.setNodeRole("ship_order", "Executor");
			
			// Check model definition
			prjSession.checkModelDefinition();
			
			System.out.println("   [ OK ]");
		} catch (Exception e) {
			System.out.println("\n\n   [ ERROR ] : " + e);
			e.printStackTrace();
		} // Maybe something is wrong
	}
}
