package hero.session;

/**
 *
 * Bonita
 * Copyright (C) 1999 Bull S.A.
 * Bull 68 route de versailles  78434 Louveciennes Cedex France
 * Further information: bonita@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along wit this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *
--------------------------------------------------------------------------
 * $Id: AdminSessionBean.java,v 1.27 2006/08/30 22:42:02 dparisek Exp $
 *
--------------------------------------------------------------------------
 */
import hero.hook.NodeHookI;
import hero.hook.ProcessHookI;
import hero.interfaces.BnProjectLocal;
import hero.interfaces.BnProjectLocalHome;
import hero.interfaces.BnProjectPK;
import hero.interfaces.BnNodeLocalHome;
import hero.interfaces.BnNodeLocal;
import hero.interfaces.BnNodePK;
import hero.interfaces.Constants;
import hero.util.EventConstants;
import hero.util.HeroException;
import hero.util.ServerType;

import java.io.File;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.HashMap;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

/**
*
* The Admin Session Bean, is an stateful session bean that provides the admin API to get information about
* projects and activities in which the user is admin.<br><br>
* <strong>The following lines shows a sample code to use this API in your application:<br><br></strong>
* <br>
* First of all you have to import the Admin Session files:<br>
* <br>
* import hero.interfaces.AdminSessionLocalHome;<br>
* import hero.interfaces.AdminSessionLocal;<br>
* import hero.interfaces.AdminSessionHome;<br>
* import hero.interfaces.AdminSession;<br>
* import hero.interfaces.AdminSessionUtil;<br>
* <br>
* Now, it is time to create the Admin Session instance:<br>
* <br>
* Like this if you want to use local interfaces:<br><br>
* AdminSessionLocalHome adminh = (AdminSessionLocalHome)hero.interfaces.AdminSessionUtil.getLocalHome();<br>
* AdminSessionLocal adminsession = adminh.create();<br>
* <br>
* or like this if you use remote interfaces:<br>  
* <br>
* AdminSessionHome adminh = (AdminSessionHome)hero.interfaces.AdminSessionUtil.getHome();<br>
* AdminSession adminsession = adminh.create();<br>
* <br>
* <br>
* Now you can call all Admin Sessions methods...  
*
 *
 * @ejb:bean name="AdminSession"
 *           display-name="AdminSession Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/hero/AdminSession"
 *           local-jndi-name="ejb/hero/AdminSession_L"
 *
 * @ejb:ejb-ref ejb-name="BnProject"
 *              ref-name="myhero/BnProject"
 * @ejb:transaction type="Supports"
 * @ejb.permission role-name="BONITAUSER,user,SuperAdmin" 
 * @jonas.bean
 *      ejb-name="AdminSession"
 *      jndi-name="ejb/hero/AdminSession"
 * 
 * @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
 **/

public class AdminSessionBean implements SessionBean, EventConstants  {

   // -------------------------------------------------------------------------
   // Static
   // -------------------------------------------------------------------------
   // Utility variable
	private static final Logger trace = Logger.getLogger(AdminSessionBean.class);
	
	private static final String SCRIPTS_DIR = ServerType.getServerBase() + File.separator + "bonitaScripts";
   
   // -------------------------------------------------------------------------
   // Members 
   // -------------------------------------------------------------------------
   
   private SessionContext mContext;
  
   // -------------------------------------------------------------------------
   // Methods
   // -------------------------------------------------------------------------  

   /**
	* Get admin workflow models (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
    *
   **/        
	   public Collection getModels() throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
	    Collection result = new ArrayList();
		BnProjectLocalHome projhome;
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();			
			Collection models =projhome.findByAdmin(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.MODEL);
			Iterator mdls = models.iterator(); 
			while (mdls.hasNext())
			{
				BnProjectLocal pj = (BnProjectLocal)mdls.next();
				result.add(pj.getBnProjectLightValue());
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	   }
	   
  /**
	* Get instances list for models in which this user is admin (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
    *
   **/        
	   public Collection getInstances() throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome projhome;
		Collection result = new ArrayList();
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection instances =projhome.findByAdmin(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
			Iterator inst = instances.iterator(); 
			while (inst.hasNext())
			{
				BnProjectLocal pj = (BnProjectLocal)inst.next();
				result.add(pj.getBnProjectLightValue());
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	   }
	   
   /**
	* Get filtered instances list for models in which this user is admin (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/ 
	   public Collection getInstances(HashMap filter) throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome projhome;
		Collection result = new ArrayList();
		try {
			Collection instances = null;
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			if (filter == null)
				instances =projhome.findByAdmin(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
			else instances = getFinderByFilter(Constants.Pj.INSTANCE, filter);

			if (instances == null) return result;
			Iterator inst = instances.iterator(); 
			while (inst.hasNext())
			{
				BnProjectLocal pj = (BnProjectLocal)inst.next();
				result.add(pj.getBnProjectLightValue());
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	   }
		  
   /**
	* Get filtered instances list for models in which this user is admin (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/ 
	   public Collection getFinderByFilter(String type, HashMap filter) throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
	   	Collection results = null;
	   	
		String filterS = "";
		boolean state = false;
		boolean creator = false;
		boolean user = false;
		boolean role = false;
		boolean property = false;
		boolean executor = false;
		boolean performer = false;
		boolean process = false;
		int instState = 0;
		String instCreator = "";
		String instUser = "";
		String instRole = "";
		String instPropertyKey = "";
		String instPropertyVal = "";
		int actState = 0;
		String actExecutor = "";
		String actPerformer = "";
		String actProcess = "";
		try {
		   	if (type.equals(Constants.Pj.INSTANCE)) {
	   			if (filter.containsKey("state")) {
	   				if (filter.get("state")!=null) {
	   					instState = cvStateToNum(filter.get("state").toString());
	   					state = true;
	   				}
	   			}
	   			if (filter.containsKey("creator")) {
	   				if (filter.get("creator")!=null) {
	   					instCreator = filter.get("creator").toString();
	   					creator = true;
	   				}
	   			}
	   			if (filter.containsKey("user")) {
	   				if (filter.get("user")!=null) {
	   					instUser = filter.get("user").toString();
	   					user = true;
	   				}
	   			}
	   			if (filter.containsKey("role")) {
	   				if (filter.get("role")!=null) {
	   					instRole = filter.get("role").toString();
	   					role = true;
	   				}
	   			}
	   			if (filter.containsKey("property")) {
	   				if (filter.get("property")!=null) {
	   					String instProperty = filter.get("property").toString();
	   					int delim = instProperty.indexOf("=");
	   					if (delim >=0) {
	   						instPropertyKey = instProperty.substring(0, delim);
	   						instPropertyVal = instProperty.substring(delim+1);
	   						if (instPropertyVal.equals("")==false) property = true;
	   					}
	   				}
	   			}
		   		BnProjectLocalHome projhome;
		   		projhome = hero.interfaces.BnProjectUtil.getLocalHome();
		   	
		   		if (state && creator && user && role && property)
		   			results =projhome.findByAdminFiltersAll(instState, instCreator, instUser, instRole, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
		   		else if (state && creator && user && role && !property)
		   			results =projhome.findByAdminFiltersNoProp(instState, instCreator, instUser, instRole, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
		   		else if (state && creator && user && !role && property)
		   			results =projhome.findByAdminFiltersNoRole(instState, instCreator, instUser, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
		   		else if (state && creator && !user && role && property)
		   			results =projhome.findByAdminFiltersNoUser(instState, instCreator, instRole, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
		   		else if (state && !creator && user && role && property)
		   			results =projhome.findByAdminFiltersNoCreator(instState, instUser, instRole, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
		   		else if (!state && creator && user && role && property)
		   			results =projhome.findByAdminFiltersNoState(instCreator, instUser, instRole, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
		   		else if (state && creator && user && !role && !property)
		   			results =projhome.findByAdminFiltersNoRoleNoProp(instState, instCreator, instUser, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
		   		else if (state && creator && !user && role && !property)
		   			results =projhome.findByAdminFiltersNoUserNoProp(instState, instCreator, instRole, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (state && creator && !user && !role && property)
					results =projhome.findByAdminFiltersNoUserNoRole(instState, instCreator, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && creator && user && role && !property)
					results =projhome.findByAdminFiltersNoStateNoProp(instCreator, instUser, instRole, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && creator && user && !role && property)
					results =projhome.findByAdminFiltersNoStateNoRole(instCreator, instUser, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (state && !creator && user && !role && property)
					results =projhome.findByAdminFiltersNoCreatorNoRole(instState, instUser, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && !creator && user && role && property)
					results =projhome.findByAdminFiltersNoStateNoCreator(instUser, instRole, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (state && creator && !user && !role && !property)
					results =projhome.findByAdminFiltersNoUserNoRoleNoProp(instState, instCreator, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (state && !creator && user && !role && !property)
					results =projhome.findByAdminFiltersNoCreatorNoRoleNoProp(instState, instUser, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (state && !creator && !user && role && !property)
					results =projhome.findByAdminFiltersNoCreatorNoUserNoProp(instState, instRole, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (state && !creator && !user && !role && property)
					results =projhome.findByAdminFiltersNoCreatorNoUserNoRole(instState, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && creator && user && !role && !property)
					results =projhome.findByAdminFiltersNoStateNoRoleNoProp(instCreator, instUser, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && creator && !user && role && !property)
					results =projhome.findByAdminFiltersNoStateNoUserNoProp(instCreator, instRole, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && creator && !user && !role && property)
					results =projhome.findByAdminFiltersNoStateNoUserNoRole(instCreator, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && !creator && user && role && !property)
					results =projhome.findByAdminFiltersNoStateNoCreatorNoProp(instUser, instRole, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && !creator && user && !role && property)
					results =projhome.findByAdminFiltersNoStateNoCreatorNoRole(instUser, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && !creator && !user && role && property)
					results =projhome.findByAdminFiltersNoStateNoCreatorNoUser(instRole, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (state && !creator && !user && !role && !property)
					results =projhome.findByAdminFiltersNoCreatorNoUserNoRoleNoProp(instState, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && creator && !user && !role && !property)
					results =projhome.findByAdminFiltersNoStateNoUserNoRoleNoProp(instCreator, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && !creator && user && !role && !property)
					results =projhome.findByAdminFiltersNoStateNoCreatorNoRoleNoProp(instUser, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && !creator && !user && role && !property)
					results =projhome.findByAdminFiltersNoStateNoCreatorNoUserNoProp(instRole, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && !creator && !user && !role && property)
					results =projhome.findByAdminFiltersNoStateNoCreatorNoUserNoRole(instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (state && !creator && !user && role && property)
					results =projhome.findByAdminFiltersNoCreatorNoUser(instState, instRole, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (state && !creator && user && role && !property)
					results =projhome.findByAdminFiltersNoCreatorNoProp(instState, instUser, instRole, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && creator && !user && role && property)
					results =projhome.findByAdminFiltersNoStateNoUser(instCreator, instRole, instPropertyKey, instPropertyVal, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
				else if (!state && !creator && !user && !role && !property)
					results =projhome.findByAdmin(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
		   	}
		   	else if (type.equals(Constants.Pj.ACTIVITY)) {
	   			if (filter.containsKey("state")) {
	   				if (filter.get("state")!=null) {
	   					actState = cvStateToNum(filter.get("state").toString());
	   					state = true;
	   				}
	   			}
	   			if (filter.containsKey("executor")) {
	   				if (filter.get("executor")!=null) {
	   					actExecutor = filter.get("executor").toString();
	   					executor = true;
	   				}
	   			}
	   			if (filter.containsKey("performer")) {
	   				if (filter.get("performer")!=null) {
	   					actPerformer = filter.get("performer").toString();
	   					performer = true;
	   				}
	   			}
	   			if (filter.containsKey("process")) {
	   				if (filter.get("process")!=null) {
	   					actProcess = filter.get("process").toString()+"_instance%";
	   					process = true;
	   				}
	   			}
	   			BnNodeLocalHome nodehome;
   				nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
	   	
		   		if (state && executor && performer && process)
		   			results =nodehome.findAdminActivitiesFilterAll(actState, actExecutor, actPerformer, actProcess, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
		   		else if (!state && executor && performer && process )
		   			results =nodehome.findAdminActivitiesFilterNoState(actExecutor, actPerformer, actProcess, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
		   		else if (!state && !executor && performer && process)
		   			results =nodehome.findAdminActivitiesFilterNoStateNoExecutor(actPerformer, actProcess, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
		   		else if (!state && !executor && !performer && process)
		   			results =nodehome.findAdminActivitiesFilterNoStateNoExecutorNoPerformer(actProcess, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
		   		else if (state && executor && performer && !process)
		   			results =nodehome.findAdminActivitiesFilterNoProcess(actState, actExecutor, actPerformer, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
		   		else if (!state && executor && performer && !process)
		   			results =nodehome.findAdminActivitiesFilterNoStateNoProcess(actExecutor, actPerformer, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
		   		else if (state && executor && !performer && process)
		   			results =nodehome.findAdminActivitiesFilterNoPerformer(actState, actExecutor, actProcess, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
		   		else if (state && !executor && !performer && !process)
		   			results =nodehome.findAdminActivitiesFilterNoExecutorNoPerformerNoProcess(actState, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
				else if (!state && executor && !performer && !process)
					results =nodehome.findAdminActivitiesFilterNoStateNoPerformerNoProcess(actExecutor, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
				else if (!state && !executor && performer && !process)
					results =nodehome.findAdminActivitiesFilterNoStateNoExecutorNoProcess(actPerformer, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
				else if (!state && executor && !performer && process)
					results =nodehome.findAdminActivitiesFilterNoStateNoPerformer(actExecutor, actProcess, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
				else if (state && !executor && performer && !process)
					results =nodehome.findAdminActivitiesFilterNoExecutorNoProcess(actState, actPerformer, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
				else if (state && !executor && !performer && process)
					results =nodehome.findAdminActivitiesFilterNoExecutorNoPerformer(actState, actProcess, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
				else if (state && executor && !performer && !process)
					results =nodehome.findAdminActivitiesFilterNoPerformerNoProcess(actState, actExecutor, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
				else if (state && !executor && performer && process)
					results =nodehome.findAdminActivitiesFilterNoExecutor(actState, actPerformer, actProcess, mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
				else if (!state && !executor && !performer && !process)
					results =nodehome.findAdminActivities(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
		   	}
	   		return results;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	   }

  /**
	* Convert alpha state string to numeric state string as used in DB
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	*/        
	  public int cvStateToNum(String state) throws HeroException {
	  trace.debug("start by " + mContext.getCallerPrincipal().getName());
	  int result = 0;
	  if (state.equals("INITIAL")) result = Constants.Nd.INITIAL;
	  else if (state.equals("STARTED")) result = Constants.Nd.READY;
	  else if (state.equals("READY")) result = Constants.Nd.READY;
	  else if (state.equals("DEAD")) result = Constants.Nd.DEAD;
	  else if (state.equals("ANTICIPABLE")) result = Constants.Nd.ANTICIPABLE;
	  else if (state.equals("EXPIRED")) result = Constants.Nd.EXPIRED;
	  else if (state.equals("ANTICIPATING")) result = Constants.Nd.ANTICIPATING;
	  else if (state.equals("EXECUTING")) result = Constants.Nd.EXECUTING;
	  else if (state.equals("EXECUTED")) result = Constants.Nd.EXECUTED;
	  else if (state.equals("INERROR")) result = Constants.Nd.INERROR;
	  else if (state.equals("FINISHED")) result = Constants.Nd.FINISHED;
	  else if (state.equals("TERMINATED")) result = Constants.Nd.TERMINATED;
	  else if (state.equals("CHECKEDOUT")) result = Constants.Nd.CHECKEDOUT;
	  else if (state.equals("ANT_SUSPENDED")) result = Constants.Nd.ANT_SUSPENDED;
	  else if (state.equals("EXEC_SUSPENDED")) result = Constants.Nd.EXEC_SUSPENDED;
	  else if (state.equals("BAD_TRANSITION")) result = Constants.Nd.BAD_TRANSITION;
	  return result;
	}
			   
   /**
	* Get filtered instances list for models for users
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/        
	   public Collection getInstanceUsers() throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome projhome;
		Collection result = new ArrayList();
		try {
			String user = null;
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection users = projhome.getModelUserList(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
			Iterator it = users.iterator(); 
			while (it.hasNext())
			{
				user = (String)it.next();
				result.add(user);
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} 
	   }
		
   /**
	* Get filtered instances list of models for creators
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/        
	   public Collection getInstanceCreators() throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome projhome;
		Collection result = new ArrayList();
		try {
			String creator = null;
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection users = projhome.getModelCreatorList(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
			Iterator it = users.iterator(); 
			while (it.hasNext())
			{
				creator = (String)it.next();
				result.add(creator);
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} 
	   }
	   
   /**
	* Get filtered instances list of models for creators
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/        
	   public Collection getInstanceRoles() throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome projhome;
		Collection result = new ArrayList();
		try {
			String role = null;
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection roles = projhome.getModelRoleList(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
			Iterator it = roles.iterator(); 
			while (it.hasNext())
			{
				role = (String)it.next();
				if (role.toLowerCase().equals("initialrole")==false) result.add(role);
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} 
	   }
		   
   /**
	* Get filtered instances list of models for states
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/        
	   public Collection getInstanceStates() throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome projhome;
		Collection result = new ArrayList();
		try {
			String state = null;
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection states = projhome.getModelStateList(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
			Iterator it = states.iterator(); 
			while (it.hasNext())
			{
				state = it.next().toString();
				switch (Integer.parseInt(state)) {
	            case (0): state = "INITIAL"; break;
	            case (1): state = "STARTED"; break;
	            case (2): state = "DEAD"; break;
	            case (3): state = "ANTICIPABLE"; break;
	            case (4): state = "EXPIRED"; break;
	            case (5): state = "ANTICIPATING"; break;
	            case (6): state = "EXECUTING"; break;
	            case (7): state = "EXECUTED"; break;
	            case (8): state = "INERROR"; break;
	            case (9): state = "FINISHED"; break;
	            case (10): state = "TERMINATED"; break;
	            case (11): state = "CHECKEDOUT"; break;
	            case (12): state = "ANT_SUSPENDED"; break;
	            case (13): state = "EXEC_SUSPENDED"; break;
	            case (14): state = "BAD_TRANSITION"; break;     
				}
				result.add(state);
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} 
	   }
		   
  /**
	* Get filtered instances list of models for property Keys
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/        
	   public Collection getInstancePropertyKeys() throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome projhome;
		Collection result = new ArrayList();
		try {
			projhome = hero.interfaces.BnProjectUtil.getLocalHome();
			Collection keys = projhome.getModelPropertyKeyList(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.MODEL);
			Iterator it = keys.iterator(); 
			while (it.hasNext())
			{
				result.add(it.next());
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} 
	   }
	   
   /**
	* Get subset instances list for models in which this user is admin (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
   **/        
	   public Collection getListAsPK(String type, HashMap filter) throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome projhome;
		BnNodeLocalHome nodehome;
		Collection result = new ArrayList();
		Collection list = null;
		Iterator it = null;
		try {
			if (type.equals(Constants.Pj.GETACTIVITIES)) {
				nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
				if (filter == null)
					list = nodehome.findAdminActivities(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
				else list = getFinderByFilter(Constants.Pj.ACTIVITY, filter);
				it = list.iterator(); 
				while (it.hasNext())
				{
					BnNodeLocal nd = (BnNodeLocal)it.next();
					result.add(nd.getPrimaryKey());
				}
			}
			else if (type.equals(Constants.Pj.GETINSTANCES) ||
					type.equals(Constants.Pj.GETMODELS)) {
				if (type.equals(Constants.Pj.GETINSTANCES)) {
					if (filter == null) {
						projhome = hero.interfaces.BnProjectUtil.getLocalHome();
						list =projhome.findByAdmin(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE);
					}
					else list = getFinderByFilter(Constants.Pj.INSTANCE, filter);
				}
				else if (type.equals(Constants.Pj.GETMODELS)) {
					projhome = hero.interfaces.BnProjectUtil.getLocalHome();
					list =projhome.findByAdmin(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.MODEL);
				}
				it = list.iterator(); 
				while (it.hasNext())
				{
					BnProjectLocal pj = (BnProjectLocal)it.next();
					result.add(pj.getPrimaryKey());
				}
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	   }
	   
   /**
	* Get subset instances list (by PK)for models in which this user is admin (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/        
	   public Collection getListByPK(String type, Collection list, int offset, int numrows) throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnProjectLocalHome projhome;
		BnNodeLocalHome nodehome;
		Collection result = new ArrayList();
		Iterator it = null;
		try {
			int add = 0;
			int cnt = 0;
			if (type.equals(Constants.Pj.GETACTIVITIES)) {
				nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
				it = list.iterator();
				while (it.hasNext()) {
					if (cnt < offset) it.next();
					else {
						BnNodeLocal nd = nodehome.findByPrimaryKey((BnNodePK)it.next());
			            result.add(nd.getBnNodeValue());
						add++;
						if (add == numrows) break;
					}
					cnt++;
				}
			}
			else if (type.equals(Constants.Pj.GETINSTANCES) ||
					type.equals(Constants.Pj.GETMODELS)) {
				projhome = hero.interfaces.BnProjectUtil.getLocalHome();
				it = list.iterator();
				while (it.hasNext()) {
					if (cnt < offset) it.next();
					else {
						BnProjectLocal pj = projhome.findByPrimaryKey((BnProjectPK)it.next());
						result.add(pj.getBnProjectLightValue());
						add++;
						if (add == numrows) break;
					}
					cnt++;
				}
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	   }
		   
   /**
	* Get activities for instances/cooperatives projects in which this user is admin (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
   **/
	   public Collection getActivities() throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection result = new ArrayList();
		try {
			nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
			Collection activities = nodehome.findAdminActivities(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
			Iterator act = activities.iterator();
			while (act.hasNext())
			{
				BnNodeLocal nd = (BnNodeLocal)act.next();
				result.add(nd.getBnNodeValue());				
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	  }	   
	   
   /**
	* Get filtered activities for instances/cooperatives projects in which this user is admin (admin is the role of the model administrator)
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/
	   public Collection getActivities(HashMap filter) throws HeroException {
	   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection result = new ArrayList();
		try {
			Collection activities = null;
			nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
			
			if (filter == null)
				activities = nodehome.findAdminActivities(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
			else activities = getFinderByFilter(Constants.Pj.ACTIVITY, filter);
			
			Iterator act = activities.iterator();
			while (act.hasNext())
			{
				BnNodeLocal nd = (BnNodeLocal)act.next();
				result.add(nd.getBnNodeValue());				
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} catch (javax.ejb.FinderException fe) {
			trace.error(fe.getMessage());
			throw new HeroException(fe.getMessage());
		}
	  }	   	
	   
   /**
	* Get filtered activities list for processes
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/        
	   public Collection getActivityProcesses() throws HeroException {
		trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection result = new ArrayList();
		try {
			String name = null;
			nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
			Collection names = nodehome.getAdminActivitiesByProcess(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.MODEL, Constants.Pj.COOPERATIVE);
			Iterator it = names.iterator(); 
			while (it.hasNext())
			{
				name = (String)it.next();
				result.add(name);
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} 
	   }
	   
   /**
	* Get filtered activities list for processes
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/        
	   public Collection getActivityState() throws HeroException {
		trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection result = new ArrayList();
		try {
			String name = null;
			nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
			Collection names = nodehome.getAdminActivitiesByState(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
			Iterator it = names.iterator(); 
			while (it.hasNext())
			{
				name = it.next().toString();
				switch (Integer.parseInt(name)) {
	            case (0): name = "INITIAL"; break;
	            case (1): name = "READY"; break;
	            case (2): name = "DEAD"; break;
	            case (3): name = "ANTICIPABLE"; break;
	            case (4): name = "EXPIRED"; break;
	            case (5): name = "ANTICIPATING"; break;
	            case (6): name = "EXECUTING"; break;
	            case (7): name = "EXECUTED"; break;
	            case (8): name = "INERROR"; break;
	            case (9): name = "FINISHED"; break;
	            case (10): name = "TERMINATED"; break;
	            case (11): name = "CHECKEDOUT"; break;
	            case (12): name = "ANT_SUSPENDED"; break;
	            case (13): name = "EXEC_SUSPENDED"; break;
	            case (14): name = "BAD_TRANSITION"; break;     
				}
				result.add(name);
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} 
	   }
		   
   /**
	* Get filtered activities list for processes
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/        
	   public Collection getActivityExecutor() throws HeroException {
		trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection result = new ArrayList();
		try {
			String name = null;
			nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
			Collection names = nodehome.getAdminActivitiesByExecutor(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
			Iterator it = names.iterator(); 
			while (it.hasNext())
			{
				name = (String)it.next();
				result.add(name);
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} 
	   }
	   
   /**
	* Get filtered activities list for processes
	* 
	* @ejb:interface-method view-type="both"
	* @ejb:transaction type="Supports"
	*
	**/        
	   public Collection getActivityPerformer() throws HeroException {
		trace.debug("start by " + mContext.getCallerPrincipal().getName());
		BnNodeLocalHome nodehome;
		Collection result = new ArrayList();
		try {
			String name = null;
			nodehome = hero.interfaces.BnNodeUtil.getLocalHome();
			Collection names = nodehome.getAdminActivitiesByPerformer(mContext.getCallerPrincipal().getName(), Constants.ADMIN, Constants.Pj.INSTANCE, Constants.Pj.COOPERATIVE);
			Iterator it = names.iterator(); 
			while (it.hasNext())
			{
				name = (String)it.next();
				result.add(name);
			}
			return result;
		} catch (javax.naming.NamingException ne) {
			trace.error(ne.getMessage());
			throw new HeroException(ne.getMessage());
		} 
	   }
	   
      /**
	    * This method retrieve the list of hooks deployed on the server
	    * 
	    * @ejb:interface-method view-type="both"
	    * @ejb:transaction type="Supports"
	    *
	    **/
	   public Collection getJavaHooks(String eventType){
		   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		   NodeHookI ndh;
		   ProcessHookI ph;
		   String event="";
		   try{
			   ArrayList names = new ArrayList();
			   String HOOKS_DIR = SCRIPTS_DIR+File.separator+"hero"+File.separator+"hook";
			   if ((new File(HOOKS_DIR)).exists())
			   {
				   File dir = new File(HOOKS_DIR);
				   Collection output = Arrays.asList(dir.list());
				   Iterator i = output.iterator();
				   while (i.hasNext())
				   {
					   String className = (String)i.next();
					   int index = className.indexOf(".class");
					   className=className.substring(0,index);
					   ClassLoader cl = Thread.currentThread().getContextClassLoader();					   
					   Class clhook=cl.loadClass("hero.hook."+className);
					   Class[] param={};
					   Object[] o={};
					   Method evth=clhook.getMethod("getMetadata",param);
					   if (NodeHookI.class.isAssignableFrom(clhook))
					   {
						   ndh =(NodeHookI)clhook.newInstance();
						   event=(String)evth.invoke(ndh,o);
					   }
					   else if (ProcessHookI.class.isAssignableFrom(clhook))
					   {
						   ph =(ProcessHookI)clhook.newInstance();
						   event=(String)evth.invoke(ph,o);
					   }					   
					   if (event.equals(eventType))
						   names.add(className);
				   }
				   return names;				   				   				   				
			   }			
			   else
				   return (new ArrayList());
			   
		   } catch (Exception e) {e.printStackTrace();return new ArrayList();
		   }        		   
	   }   
	   
	   /**
	    * This method retrieve the list of mappers deployed on the server
	    * 
	    * @ejb:interface-method view-type="both"
	    * @ejb:transaction type="Supports"
	    *
	    **/
	   public Collection getCustomMappers() throws HeroException {
		   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		   
		   try{
			   ArrayList names = new ArrayList();
			   String MAPPER_DIR = SCRIPTS_DIR+File.separator+"hero"+File.separator+"mapper";
			   if ((new File(MAPPER_DIR)).exists())
			   {
				   File dir = new File(MAPPER_DIR);
				   Collection output = Arrays.asList(dir.list());
				   Iterator i = output.iterator();
				   while (i.hasNext())
				   {
					   String className = (String)i.next();
					   int index = className.indexOf(".class");
					   names.add(className.substring(0,index));
				   }
				   return names;				   				   				   				
			   }			
			   else
				   return (new ArrayList());
			   
		   } catch (Exception e) {
			   throw new HeroException(e.getMessage());
		   }        		   
	   }   
	  
	   /**
	    * This method retrieve the list of performer assignments deployed on the server
	    * 
	    * @ejb:interface-method view-type="both"
	    * @ejb:transaction type="Supports"
	    *
	    **/
	   public Collection getCallbackPerformers() throws HeroException {
		   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		   
		   try{
			   ArrayList names = new ArrayList();
			   String PERFORMER_DIR = SCRIPTS_DIR+File.separator+"hero"+File.separator+"performerAssign";
			   if ((new File(PERFORMER_DIR)).exists())
			   {
				   File dir = new File(PERFORMER_DIR);
				   Collection output = Arrays.asList(dir.list());
				   Iterator i = output.iterator();
				   while (i.hasNext())
				   {
					   String className = (String)i.next();
					   int index = className.indexOf(".class");
					   names.add(className.substring(0,index));
				   }
				   return names;				   				   				   				
			   }			
			   else
				   return (new ArrayList());
			   
		   } catch (Exception e) {
			   throw new HeroException(e.getMessage());
		   }        		   
	   }   
	   
	   /**
	    * This method retrieve the list of Initiator Mappers deployed on the server
	    * 
	    * @ejb:interface-method view-type="both"
	    * @ejb:transaction type="Supports"
	    *
	    **/
	   public Collection getCustomInitiators() throws HeroException {
		   trace.debug("start by " + mContext.getCallerPrincipal().getName());
		   
		   try{
			   ArrayList names = new ArrayList();
			   String INITIATOR_DIR = SCRIPTS_DIR+File.separator+"hero"+File.separator+"initiatorMapper";
			   if ((new File(INITIATOR_DIR)).exists())
			   {
				   File dir = new File(INITIATOR_DIR);
				   Collection output = Arrays.asList(dir.list());
				   Iterator i = output.iterator();
				   while (i.hasNext())
				   {
					   String className = (String)i.next();
					   int index = className.indexOf(".class");
					   names.add(className.substring(0,index));
				   }
				   return names;				   				   				   				
			   }			
			   else
				   return (new ArrayList());
			   
		   } catch (Exception e) {
			   throw new HeroException(e.getMessage());
		   }        		   
	   }   
	   
   /**
   * Create the AdminSession Bean
   *
   * @throws CreateException 
   *
   * @ejb.permission unchecked="yes" 
   * @ejb:create-method view-type="both"
   **/

    public void ejbCreate() throws CreateException {
	trace.debug("ejbCreate");
    }
    
    public void setSessionContext(final javax.ejb.SessionContext context) {
	mContext = context;
    }
    
    public void ejbRemove() {
    }
    
    public void ejbActivate() {
    }
    
    public void ejbPassivate() {
    }
    
}
