package hero.session;

import hero.user.ReadEnv;
import hero.util.BonitaConfig;
import hero.util.EventConstants;
import hero.util.HeroException;
import java.util.Vector;

import hero.historic.ProjectHistoric;
import hero.historic.NodeHistoric;
import hero.historic.PropertyHistoric;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

import org.apache.xerces.parsers.DOMParser;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.NodeList;

/**
*
* The History Session Bean, is an stateless session bean that provides the user API to get information on
* historic instance and activity properties.<br>
*<br><br>
* <strong>The following lines shows a sample code to use this API in your application:<br><br></strong>
* <br>
* First of all you have to import the History Session files:<br>
* <br>
* import hero.interfaces.HistorySessionLocalHome;<br>
* import hero.interfaces.HistorySessionLocal;<br>
* import hero.interfaces.HistorySessionHome;<br>
* import hero.interfaces.HistorySession;<br>
* import hero.interfaces.HistorySessionUtil;<br>
* <br>
* Now, it is time to create the History Session instance:<br>
* <br>
* Like this if you want to use local interfaces:<br><br>
* HistorySessionLocalHome historyh = (HistorySessionLocalHome)hero.interfaces.HistorySessionUtil.getLocalHome();<br>
* HistorySessionLocal historysession = historyh.create();<br>
* <br>
* or like this if you use remote interfaces:<br>  
* <br>
* HistorySessionHome historyh = (HistorySessionHome)hero.interfaces.HistorySessionUtil.getHome();<br>
* HistorySession historysession = historyh.create();<br>
* <br>
* <br>
* Now you can call all History Sessions methods...  
* 
*
* @ejb:bean name="HistorySession"
*           display-name="HistorySession Bean"
*           type="Stateless"
*           transaction-type="Container"
*           jndi-name="ejb/hero/HistorySession"
*           local-jndi-name="ejb/hero/HistorySession_L"
*
* @ejb.permission role-name="BONITAUSER,user,SuperAdmin" 
*
*
* @jonas.bean
*      ejb-name="HistorySession"
*      jndi-name="ejb/hero/HistorySession"
*
* @jboss.container-configuration name="Standard Stateless SessionBean for Bonita"
* 
* @copyright (C) 2007 Bull S.A.
* @author D. Parisek
**/

public class HistorySessionBean implements SessionBean, EventConstants {
	
	// -------------------------------------------------------------------------
	// Static
	// -------------------------------------------------------------------------
	// Utility variable
	private static final Logger trace = Logger.getLogger(HistorySessionBean.class);
	
	// -------------------------------------------------------------------------
	// Members 
	// -------------------------------------------------------------------------
	
	private SessionContext mContext;
	private BonitaConfig bonitaConfig;
	
	// -------------------------------------------------------------------------
	// Methods
	// -------------------------------------------------------------------------
	
	/**
	 * Get list of project histories by project name w/o pagination.
	 * @return a Collection of projects that have history
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getHistoryProjectList() throws HeroException {
		try {
			return (getHistoryProjectList(0, 0));
		} catch (Exception e) {
			throw new HeroException("Error getting history projects: " + e.getMessage());
		}
	}
	
	/**
	 * Get list of project histories by project name w/ pagination.
	 * @return a Collection of projects that have history
	 * @param startindex
	 * @param nrows
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getHistoryProjectList(int startindex, int nrows) throws HeroException {
		ArrayList historyProjects = new ArrayList();
		try{
			String historyDir = initHistoryLocation();
			if (historyDir.equals("")) return null;
		    File historicDirectory = new File(historyDir);
			trace.debug("start by " + mContext.getCallerPrincipal().getName());
			
	        if ( historicDirectory.isDirectory() )
	        {
	           File[] processDirectories = historicDirectory.listFiles();
	           int limit = processDirectories.length;
	           if (nrows == 0) startindex = 0;
	           else if ((startindex+nrows) < limit) limit = startindex+nrows;
	           for ( int i = startindex; i < limit; i++)
	           {
	              File processDirectory = processDirectories[i];
	              String processName = processDirectory.getName();
	              historyProjects.add(processName);
	           }
	        }
	        return historyProjects;
		} catch (Exception e) {
			throw new HeroException("Error getting history projects: " + e.getMessage());
		}
	}
	
	/**
	 * Get instance history attributes.
	 * @return a ProjectHistoric object - get project attrs
	 * @parsm projectName
	 * @param instanceName
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Object getHistoryInstance(String instanceName) throws HeroException {
		try{
			int delim = instanceName.indexOf("_instance");
			if(delim<0) return null;
			String projectName = instanceName.substring(0, delim);
			Vector nodelist = new Vector();
			String historyDir = initHistoryLocation();
			if (historyDir.equals("")) return null;
			String instancePath = historyDir + File.separator + projectName + File.separator + instanceName + ".xml";

			ProjectHistoric ph = new ProjectHistoric();
	
			DOMParser parser = new DOMParser();
			parser.parse(instancePath);
			Document document = parser.getDocument();
			Element element = document.getDocumentElement();
			if(element.hasAttributes()) {
				ph.setName(element.getAttribute("name"));
				ph.setVersion(element.getAttribute("version"));
				ph.setCreationDate(element.getAttribute("creationDate"));
				ph.setEndDate(element.getAttribute("endDate"));
				ph.setInitiator(element.getAttribute("initiator"));
			}
			// set nodes (activities)
			NodeList nodes = document.getElementsByTagName("Node");
			for (int i=0; i<nodes.getLength(); i++) {
				NodeHistoric nh = new NodeHistoric();
				Element nodeElement = (Element)nodes.item(i);
				if(nodeElement.hasAttributes()) {
					nh.setName(nodeElement.getAttribute("name"));
					nh.setState(nodeElement.getAttribute("state"));
					nh.setRole(nodeElement.getAttribute("role"));
					nh.setExecutor(nodeElement.getAttribute("executor"));
					nh.setType(nodeElement.getAttribute("type"));
					if(nodeElement.getAttribute("anticipable").equals("true"))
						nh.setAnticipable(true);
					else nh.setAnticipable(false);
					nh.setDescription(nodeElement.getAttribute("description"));
					nh.setStartDate(nodeElement.getAttribute("startDate"));
					nh.setEndDate(nodeElement.getAttribute("endDate"));
					nodelist.add(nh);
				}
				// set nodes (activities) properties
				Vector nproplist = new Vector();
				NodeList nprops = nodeElement.getElementsByTagName("Property");
				for (int i2=0; i2<nprops.getLength(); i2++) {
					PropertyHistoric pph = new PropertyHistoric();
					Element propElement = (Element)nprops.item(i2);
					if(propElement.hasAttributes()) {
						pph.setKey(propElement.getAttribute("key"));
						pph.setValue(propElement.getAttribute("value"));
						nproplist.add(pph);
					}
				}
				if (nproplist.size()>0) nh.setProperties(nproplist);
				else nh.setProperties(null);
			}
			if (nodelist.size()>0) ph.setNodes(nodelist);
			else ph.setNodes(null);
			
			// set project properties
			Vector pproplist = new Vector();
			NodeList pprops = element.getElementsByTagName("Property");
			for (int i=0; i<pprops.getLength(); i++) {
				PropertyHistoric pph = new PropertyHistoric();
				Element propElement = (Element)pprops.item(i);
				if(propElement.getParentNode().getNodeName().equals("Instance") &&
						propElement.hasAttributes()) {
					pph.setKey(propElement.getAttribute("key"));
					pph.setValue(propElement.getAttribute("value"));
					pproplist.add(pph);
				}
			}
			if (pproplist.size()>0) ph.setProperties(pproplist);
			else ph.setProperties(null);
			return(ph);
		} catch (Exception e) {
			throw new HeroException("Error getting history instance properties: " + e.getMessage());
		}
	}
	
	private Vector getHistoryNodes(String instanceName, String instancePath) throws HeroException {
		try{
			Vector result = new Vector();
			NodeHistoric nh = new NodeHistoric();
			
			DOMParser parser = new DOMParser();
			parser.parse(instancePath);
			Document document = parser.getDocument();
			NodeList nodes = document.getElementsByTagName("Instance");
			for (int i=0; i<nodes.getLength(); i++) {
				Element nodeElement = (Element)nodes.item(i);
				nh.setName(nodeElement.getAttribute("name"));
				nh.setState(nodeElement.getAttribute("state"));
				nh.setRole(nodeElement.getAttribute("role"));
				nh.setExecutor(nodeElement.getAttribute("executor"));
				nh.setType(nodeElement.getAttribute("type"));
				if(nodeElement.getAttribute("anticipable").equals("true"))
					nh.setAnticipable(true);
				else nh.setAnticipable(false);
				nh.setDescription(nodeElement.getAttribute("description"));
				nh.setStartDate(nodeElement.getAttribute("startDate"));
				nh.setEndDate(nodeElement.getAttribute("endDate"));
				result.add(nh);
			}
			return (result);
		} catch (Exception e) {
			throw new HeroException("Error getting history instance activity nodes: " + e.getMessage());
		}
	} 
	
	/**
	 * Get instance activity history list.
	 * @return a Collection of activity names
	 * @param projectName
	 * @param instanceName
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getHistoryInstanceActivityList(String instanceName) throws HeroException {
		ArrayList nodeList = new ArrayList();
		try{
			int delim = instanceName.indexOf("_instance");
			if(delim<0) return null;
			String projectName = instanceName.substring(0, delim);
			String historyDir = initHistoryLocation();
			if (historyDir.equals("")) return null;
			String instancePath = historyDir + File.separator + projectName + File.separator + instanceName + ".xml";
			
			DOMParser parser = new DOMParser();
			parser.parse(instancePath);
			Document document = parser.getDocument();
			NodeList nodes = document.getElementsByTagName("Node");
			for (int i=0; i<nodes.getLength(); i++) {
				Element nodeElement = (Element)nodes.item(i);
				nodeList.add(nodeElement.getAttribute("name"));
			}
			return (nodeList);
		} catch (Exception e) {
			throw new HeroException("Error getting history instance activity list: " + e.getMessage());
		}
	}
	
	/**
	 * Get instance activity history list w/o pagination.
	 * @return a Collection of activity names
	 * @param projectName
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getHistoryInstancesList(String projectName) throws HeroException {
		try {
			return getHistoryInstancesList(projectName, 0, 0);
		} catch (Exception e) {
			throw new HeroException("Error getting history instance activity list: " + e.getMessage());
		}
	}
		
	/**
	 * Get instance activity history list w/ pagination.
	 * @return a Collection of activity names
	 * @param projectName
	 * @param startindex
	 * @param nrows
	 * @throws HeroException
	 * @ejb:interface-method view-type="both"
	 * @ejb:transaction type="Supports"
	 *
	 **/
	public Collection getHistoryInstancesList(String projectName, int startindex, int nrows) throws HeroException {
		ArrayList instList = new ArrayList();
		try{
			String historyDir = initHistoryLocation();
			if (historyDir.equals("")) return null;
			String instancePath = historyDir + File.separator + projectName;
	      	File processDirectory = new File(instancePath);
	      	if ( processDirectory.isDirectory ( ) ) {
	      		File[] instancesFiles = processDirectory.listFiles();
	      		int limit = instancesFiles.length;
		        if (nrows == 0) startindex = 0;
		        else if ((startindex+nrows) < limit) limit = startindex+nrows;
	            for ( int i = startindex; i < limit; i++) {
	               File instanceFile = instancesFiles[i];
	               String instanceName = instanceFile.getName();
	               instList.add(instanceName);
	            }
	      	}
			return (instList);
		} catch (Exception e) {
			throw new HeroException("Error getting history instance activity list: " + e.getMessage());
		}
	}
	
	private String initHistoryLocation() throws HeroException {
	   String HISTORIC_DIR = "";
	   try
	   {
  	      ReadEnv renv = new ReadEnv();
		  HISTORIC_DIR = renv.getVariable("BONITA_HOME")+File.separator+ "bonita-historic";

	      if (!(new File(HISTORIC_DIR)).exists()) return ("");
	   }
	   catch (Exception e)
	   {
		   throw new HeroException("Error getting history instance activity list: " + e.getMessage());
	   }
	   return (HISTORIC_DIR);
	}
	
	/**
	 * Creates the History Session Bean. This method is the first one to invoke in order to
	 * use HistorySession API. If the user is not authorized this method throws an exception.
	 *
	 * @throws CreateException
	 *
	 * @ejb:create-method view-type="both"
	 **/
	public void ejbCreate() throws CreateException {	
	}
	
	/**
	 * Internal Enterprise Java Beans method. 
	 **/
	
	public void setSessionContext(final javax.ejb.SessionContext context) {
		mContext = context;
	}
	
	/**
	 * Internal Enterprise Java Beans method. 
	 **/
	public void ejbRemove() {
	}
	
	/**
	 * Internal Enterprise Java Beans method. 
	 **/
	public void ejbActivate() {
	}
	
	/**
	 * Internal Enterprise Java Beans method. 
	 **/
	public void ejbPassivate() {
	}
}